package com.pcsilva.adc.article1.mxml.services
{
	import com.pcsilva.adc.article1.mxml.entity.Group;
	import com.pcsilva.adc.article1.mxml.entity.User;
	import com.pcsilva.util.ObjectUtil;
	
	import mx.collections.ArrayCollection;
	import mx.controls.Alert;
	import mx.rpc.events.FaultEvent;
	import mx.rpc.events.ResultEvent;
	import mx.rpc.remoting.Operation;
	import mx.rpc.remoting.RemoteObject;
	
	public class UserService
	{
		private var _remoteObject:RemoteObject;
		
		/* it perfoms ownership  lock transaction */
		private var _inUse:Boolean = false;
		
		private static var _instance:UserService;
		
		/* constant configuration remoting */
		public static const DESTINATION:String           = "ColdFusion";
		public static const SOURCE:String                = "com.pcsilva.adc.article1.cfml.facade.UserFacade";
		public static const SHOWBUSYCURSOR:Boolean       = true;
		
		/* constants with names of methods for transactions */
		public static const OPERATION_LIST_GROUP:String  = 'getGroups';
		public static const OPERATION_LIST_USER:String   = 'getUsersByGroup';
		public static const OPERATION_SAVE_USER:String   = 'saveUser';

		/* constant system, property names in VO */
		public static const LABELFIELD_GROUP:String      = 'name';		
		public static const LABELFIELD_USER:String       = 'fullname';	
		
		/* constants with texts for the VIEW */
		public static const TITLE:String                 = 'ColdFusion ORM Flex';
		
		public static const LIST_GROUPS:String           = 'Groups';		
		public static const LIST_USERS:String            = 'Users';
		
		public static const LABEL_USER:String            = 'User';		
		public static const LABEL_USER_NAME:String       = 'Name';		
		public static const LABEL_USER_LASTNAME:String   = 'Last name';		
		public static const LABEL_USER_EMAIL:String      = 'E-mail';		
		public static const LABEL_GROUP:String           = 'Group';		
		
		public static const MESSAGE_ERROR_TITLE:String   = 'Error';
		public static const MESSAGE_ERROR_CONTENT:String = 'Invalid user!';
		
		/* singleton data for VIEW */
		[Bindable] public var user:User;
		[Bindable] public var group:Group;
		[Bindable] public var users:ArrayCollection;
		[Bindable] public var groups:ArrayCollection;
		

		/* system methods */
		public function clean(instance:SingletonUserService=null):void
		{
			users  = new ArrayCollection();
			user   = new User();
			if(instance != null)
			{
				groups = new ArrayCollection();	
			}
		}
		
		protected function onFault(evt:FaultEvent):void 
		{
			_inUse = false; 
			Alert.show(evt.fault.message);                 
		}

		
		/* start constructor */
		public function UserService(instance:SingletonUserService)
		{
			if(instance != null)
			{
				clean(instance);
				_remoteObject                 = new RemoteObject();
				_remoteObject.destination     = DESTINATION;
				_remoteObject.source          = SOURCE;
				_remoteObject.showBusyCursor  = SHOWBUSYCURSOR;
				_remoteObject.addEventListener(FaultEvent.FAULT, onFault);
			}
		}
		public static function getInstance():UserService
		{
			if(_instance == null)
			{
				_instance = new UserService(new SingletonUserService());
			}
			return _instance;
		}
		/* end constructor */

		/* start getGroups */
		public function getGroups(result:Function=null):void{
			clean(new SingletonUserService());
			/* I capture operation in UserFacade, without dirtying the base configuration of remoting in Flex */
			var service:Operation = Operation(_remoteObject.getOperation(OPERATION_LIST_GROUP));
				service.addEventListener(ResultEvent.RESULT,onGetGroups);
			if(result != null)
			{
				service.addEventListener(ResultEvent.RESULT,result);
			}
			service.send(null);
		}
		protected function onGetGroups(evt:ResultEvent):void {
			groups = ArrayCollection(ObjectUtil.copy(evt.result));		
		}
		/* end getGroups */
		
		/* start getUsersByGroup */
		public function getUsersByGroup(_group:Group,result:Function=null):void{
			clean();
			var service:Operation = Operation(_remoteObject.getOperation(OPERATION_LIST_USER));
				service.addEventListener(ResultEvent.RESULT,onUsersByGroup);
			if(result != null)
			{
				service.addEventListener(ResultEvent.RESULT,result);
			}
			service.send(_group); 
		}
		protected function onUsersByGroup(evt:ResultEvent):void {
			users = ArrayCollection(ObjectUtil.copy(evt.result));		
		}
		/* end getUsersByGroup */

		/* start saveUser */
		public function saveUser(result:Function=null,fault:Function=null):void{
			/* it perfoms ownership  lock transaction */
			if(!_inUse){
				_inUse = true;
				var	_user:User = user;
				var service:Operation = Operation(_remoteObject.getOperation(OPERATION_SAVE_USER));
					service.addEventListener(ResultEvent.RESULT,onSaveUser);
					if(result != null)
					{
						service.addEventListener(ResultEvent.RESULT,result);
					}
					if(fault != null)
					{
						service.addEventListener(FaultEvent.FAULT, fault);
					}
				service.send(_user);
				user = new User();			
			}
		}
		protected function onSaveUser(evt:ResultEvent):void {
			_inUse = false;		
		}
		/* end saveUser */
	}
}
class SingletonUserService {}